print("import: 'metakernel'")
import metakernel

print("import: 'metakernel.magics'")
import metakernel.magics

print("import: 'metakernel.magics.tests'")
import metakernel.magics.tests

print("import: 'metakernel.tests'")
import metakernel.tests

print("import: 'metakernel.utils'")
import metakernel.utils

